/*
 * tmos - page.c
 *
 * Page allocator
 *
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <limits.h>

/*
 * bitmap of used pages
 */
static unsigned char __xdata page_bitmap[NUM_PAGES / CHAR_BIT];

static inline unsigned char __ffsb(unsigned char x) __reentrant
{
	unsigned char num = 0;

	if ((x & 0xf) == 0) {
		num += 4;
		x >>= 4;
	}
	if ((x & 0x3) == 0) {
		num += 2;
		x >>= 2;
	}
	if ((x & 1) == 0)
		num += 1;
	return num;
}

#define ffzb(x) __ffsb(~(x))

/**
 * page_alloc - allocate page
 *
 * This function return page identifier; PAGE_NULL on error.
 */
page_t page_alloc(void) __reentrant
{
	page_t page = PAGE_NULL;
	unsigned char i;

	preempt_disable();
	for (i = 0; i < ARRAY_SIZE(page_bitmap); i++) {
		if (page_bitmap[i] != 0xff) {
			page = ffzb(page_bitmap[i]);
			page_bitmap[i] |= 1 << page;
			page += i << 3;
			break;
		}
	}
	preempt_enable();
	return page;
}

/**
 * page_free - free page allocated by alloc_page()
 * @page: page identifier
 */
void page_free(page_t page) __reentrant
{
	preempt_disable();
	page_bitmap[page / CHAR_BIT] &= ~(1 << (page % CHAR_BIT));
	preempt_enable();
}
