/*
 * aduc - spi.c
 * 	version 0.0.0
 * 
 * Copyright (C) 2009  Marek Piwonski <marek@microster.pl>
 * Copyright (C) 2009, 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/spi.h>
#include <tmos/sched.h>
#include <asm/aduc841.h>
#include <tmos/port.h>
#include <asm/irqs.h>

#define SPICON_ISPI	0x80	/* SPI - Interrupt bit */
#define SPICON_WCOL	0x40	/* Write collision error bit */
#define SPICON_SPE	0x20	/* SPI interface enable bit */
#define SPICON_SPIM	0x10	/* Master = 1 Slave = 0 */
#define SPICON_CPOL	0x08	/* CLOCK polarity select 1= idle HIGH */
#define SPICON_CPHA	0x04	/* Clock Phase select bit */
#define SPICON_DIV2	0x00	/* fosc/2 */
#define SPICON_DIV4	0x01	/* fosc/4 */
#define SPICON_DIV8	0x02	/* fosc/8 */
#define SPICON_DIV16	0x03	/* fosc/16 */

#ifdef CONFIG_SPI_ADUC841_MASTER
#define SPI_MODE	SPICON_SPIM
#else
#define SPI_MODE	0
#endif

#ifdef CONFIG_SPI_ADUC841_CPOL
#define SPI_POL		SPICON_CPOL
#else
#define SPI_POL		0
#endif

#ifdef CONFIG_SPI_ADUC841_CPHA
#define SPI_PHA		SPICON_CPHA
#else
#define SPI_PHA		0
#endif

#if CONFIG_SPI_ADUC841_DIV2
#define SPE_SPEED	SPICON_DIV2
#elif CONFIG_SPI_ADUC841_DIV4
#define SPE_SPEED	SPICON_DIV4
#elif CONFIG_SPI_ADUC841_DIV8
#define SPE_SPEED	SPICON_DIV8
#elif CONFIG_SPI_ADUC841_DIV16
#define SPE_SPEED	SPICON_DIV16
#else
#error "unsupported speed"
#endif

#define SPI_CNF	(SPI_MODE | SPICON_SPE | SPI_POL | SPI_PHA | SPE_SPEED)

uint8_t spi_xmit(uint8_t dane)
{
	uint8_t tmp;

	SPIDAT = dane;
	while (!ISPI) {
//		schedule();
	}
	WCOL = 0;
	ISPI = 0;
	tmp = SPIDAT;
	return tmp;
}

void spi_start(u8 c) __reentrant
{
	WCOL = 0;
	ISPI = 0;
	SPIDAT = c;
}

u8 spi_send(u8 c) __reentrant
{
	u8 ret;

	while (!ISPI)
		cpu_relax();
	ISPI = 0;

	ret = SPIDAT;
	SPIDAT = c;
	return ret;
}

u8 spi_stop(void) __reentrant
{
	while (!ISPI)
		cpu_relax();
	ISPI = 0;
	return SPIDAT;
}

static void spi_init(void)
{
	SPICON = SPI_CNF;
}
device_initcall(spi_init)
